/*
 * iTunes Signature Maker
 * copyright (c) 2005 by Jason Freeman
 * 
 * For more information, please visit http://www.jasonfreeman.net/itsm/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 */

package net.jasonfreeman.itsm.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import quicktime.QTException;
import quicktime.app.time.TaskAllMovies;
import quicktime.io.OpenMovieFile;
import quicktime.io.QTFile;
import quicktime.std.StdQTException;
import quicktime.std.movies.Movie;

import net.jasonfreeman.itsm.ITSM;
import net.jasonfreeman.itsm.util.ErrorHandler;
import net.jasonfreeman.itsm.util.QTSessionManager;
import net.jasonfreeman.itsm.util.SettingsManager;

/** 
 * Enable signature playback and show other options once signature is created.
 * 
 * @author jason
 *
 */

public class FinishedPanel extends JPanel implements ActionListener, ChangeListener {

	JButton playButton, restartButton, shareButton, structureButton, revealButton;
	
	JSlider volSlider;

	Movie mov = null;

	boolean buttonSaysPlay = true;

	boolean stopMovieCheckingThread;

	ITSM applet;

	boolean tasking = false;
	
	boolean shutDownApplet = false;
	
	boolean movieCheckingThreadRunning = false;

	public FinishedPanel(ITSM applet) {
		super();
		this.applet = applet;
		setBackground(Color.white);
		SettingsManager settings = SettingsManager.getInstance();
		
		OpenMovieFile omf;
		try {
			QTSessionManager.getInstance();
			QTFile qtf = new QTFile(settings.getOutFile());
			omf = OpenMovieFile.asRead(qtf);
			mov = Movie.fromFile(omf);
			omf.close();
		} catch (QTException e) {
			ErrorHandler.getInstance().handleError(e, "Something weird happened: we can't open the signature file that we just saved to disk, " +
					"so you won't be able to play it from here. If you can find the file on your hard disk, you may be able to open and play it directly.");
		}

		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		add(Box.createVerticalGlue());
		add(Box.createVerticalGlue());

		JPanel textPanel = new JPanel();
		textPanel.setBackground(Color.white);
		textPanel.setLayout(new BoxLayout(textPanel, BoxLayout.X_AXIS));
		textPanel.add(Box.createHorizontalGlue());
		JLabel label = new JLabel(
				"iTSM has created your signature and saved it in:");
		label.setFont(label.getFont().deriveFont(Font.BOLD));
		textPanel.add(label);
		textPanel.add(Box.createHorizontalGlue());
		add(textPanel);
		add(Box.createVerticalGlue());

		JPanel filePanel = new JPanel();
		filePanel.setBackground(Color.white);
		filePanel.setLayout(new BoxLayout(filePanel, BoxLayout.X_AXIS));
		filePanel.add(Box.createHorizontalGlue());
		label = new JLabel(settings.getOutFile());
		filePanel.add(label);
		filePanel.add(Box.createHorizontalGlue());
		add(filePanel);
		add(Box.createVerticalGlue());
		add(new HorizontalSeparator());
		add(Box.createVerticalGlue());

		JPanel buttonPanel = new JPanel();
		buttonPanel.setBackground(Color.white);
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
		buttonPanel.add(Box.createHorizontalGlue());
		playButton = new JButton("play my signature");
		playButton.addActionListener(this);
		playButton.setBackground(Color.white);
		buttonPanel.add(playButton);
		buttonPanel.add(Box.createHorizontalGlue());
		add(buttonPanel);
		add(Box.createRigidArea(new Dimension(0,5)));
		
		JPanel volPanel = new JPanel();
		volPanel.setBackground(Color.white);
		volPanel.setLayout(new BoxLayout(volPanel, BoxLayout.X_AXIS));
		volPanel.add(Box.createHorizontalGlue());
		label = new JLabel("volume:");
		volPanel.add(label);
		volPanel.add(Box.createRigidArea(new Dimension(10, 0)));
	    volSlider = new JSlider(0, 150, 100);
	    volSlider.setBackground(Color.white);
	    volSlider.setPaintTicks(false);
	    volSlider.setPaintLabels(false);
	    volSlider.setSnapToTicks(false);
	    volSlider.addChangeListener(this);
	    volPanel.add(volSlider);
	    volPanel.add(Box.createHorizontalGlue());
	    add(volPanel);
		
		add(Box.createVerticalGlue());
		add(Box.createVerticalGlue());

		JPanel sharePanel = new JPanel();
		sharePanel.setBackground(Color.white);
		sharePanel.setLayout(new BoxLayout(sharePanel, BoxLayout.X_AXIS));
		sharePanel.add(Box.createHorizontalGlue());
		shareButton = new JButton("share my signature");
		shareButton.addActionListener(this);
		shareButton.setBackground(Color.white);
		sharePanel.add(shareButton);
		sharePanel.add(Box.createHorizontalGlue());
		
		structureButton = new JButton("show signature structure");
		structureButton.addActionListener(this);
		structureButton.setBackground(Color.white);
		sharePanel.add(structureButton);
		sharePanel.add(Box.createHorizontalGlue());

		String revealText = "reveal signature in Finder";
		if (System.getProperty("os.name").startsWith("Windows"))
			revealText = "reveal signature in Explorer";
		revealButton = new JButton(revealText);
		revealButton.addActionListener(this);
		revealButton.setBackground(Color.white);
		sharePanel.add(revealButton);
		sharePanel.add(Box.createHorizontalGlue());
		
		add(sharePanel);
		add(Box.createVerticalGlue());
		
		sharePanel = new JPanel();
		sharePanel.setBackground(Color.white);
		sharePanel.setLayout(new BoxLayout(sharePanel, BoxLayout.X_AXIS));
		sharePanel.add(Box.createHorizontalGlue());
		
		restartButton = new JButton("restart iTSM");
		restartButton.addActionListener(this);
		restartButton.setBackground(Color.white);
		sharePanel.add(restartButton);
		sharePanel.add(Box.createHorizontalGlue());
		
		add(sharePanel);
		add(Box.createVerticalGlue());
		add(new HorizontalSeparator());
		add(Box.createVerticalGlue());

		textPanel = new JPanel();
		textPanel.setBackground(Color.white);
		textPanel.setLayout(new BoxLayout(textPanel, BoxLayout.X_AXIS));
		textPanel.add(Box.createHorizontalGlue());
		label = new JLabel("Thanks for using iTunes Signature Maker!");
		label.setFont(label.getFont().deriveFont(Font.BOLD));
		textPanel.add(label);
		textPanel.add(Box.createHorizontalGlue());
		add(textPanel);
		add(Box.createVerticalGlue());
		add(Box.createVerticalGlue());
		applet.setFinishedPanel(this);
		applet.setContentPane(this);
		applet.validate();
	}

	/**
	 * Close movie and QT engine when we're done with this panel.
	 *
	 */
	public void stopMovie() {
		try {
			shutDownApplet = true;
			mov.stop();
			if (movieCheckingThreadRunning)
				stopMovieCheckingThread = true;
			else
				QTSessionManager.shutdown();
		} catch (Exception e2) {
			// do nothing, we're restarting anyways!
		}
	}
	
	public void actionPerformed(ActionEvent e) {
		try {
			if (e.getSource() == playButton) {
				if (buttonSaysPlay) {
					if (mov == null) {
						ErrorHandler.getInstance().handleWarning("We were unable to load your signature from disk, so we cannot play it.");
						return;
					}
					mov.setTimeValue(0);
					if (!tasking) {
						TaskAllMovies.addMovieAndStart();
						tasking = true;
					}
					mov.start();
					playButton.setText("stop");
					buttonSaysPlay = false;
					startMovieCheckingThread();
				} else {
					mov.stop();
					playButton.setText("play my signature");
					stopMovieCheckingThread = true;
					buttonSaysPlay = true;
				}
			} else if (e.getSource() == restartButton) {
				stopMovie();
				applet.restart();
			} else if (e.getSource() == shareButton) {
				applet.share();
			} else if (e.getSource() == structureButton) {
				applet.showStructure();
			} else if (e.getSource() == revealButton) {
				File outFile = new File(SettingsManager.getInstance().getOutFile());
				String cmd = outFile.getParent();
				if (System.getProperty("os.name").startsWith("Windows")) {
					cmd = "explorer " + cmd;
				} else { // we're on Mac
					cmd = "open " + cmd;
				}
				Runtime.getRuntime().exec(cmd);	
			}
		} catch (Throwable e1) {
			ErrorHandler.getInstance().handleError(e1);
		}
	}

	/**
	 * Monitor movie and maintain play/stop button, tasking thread, and
	 * QT Session as needed.
	 *
	 */
	private void startMovieCheckingThread() {
		stopMovieCheckingThread = false;
		Runnable t = new Runnable() {

			public void run() {
				movieCheckingThreadRunning = true;
				try {
					while (!mov.isDone()) {
						try {
							Thread.sleep(500);
						} catch (InterruptedException e) {
						}
						if (stopMovieCheckingThread) {
							if (shutDownApplet)
								QTSessionManager.shutdown();
							movieCheckingThreadRunning = false;
							return;
						}
					}
				} catch (Throwable e) {
					ErrorHandler.getInstance().handleError(e);
				}
				Runnable u = new Runnable() {
					public void run() {
						try {
							mov.stop();
						} catch (StdQTException e) {
							// ignore
						}
						try {
							playButton.setText("play my signature");
							buttonSaysPlay = true;
						} catch (Throwable e) {
							ErrorHandler.getInstance().handleError(e);
						}
					}
				};
				SwingUtilities.invokeLater(new Thread(u));
				movieCheckingThreadRunning = false;
			}

		};
		new Thread(t).start();
	}

	public void stateChanged(ChangeEvent e) {
	    try {
			JSlider s = (JSlider) e.getSource();
			if (s == volSlider) { 
					if (mov == null)
						return;
					int w = (int)s.getValue();
					try {
						mov.setVolume(w * 0.01f);
					} catch (StdQTException e1) {
						ErrorHandler.getInstance().handleError(e1);
					}
			}
		} catch (Throwable e1) {
			ErrorHandler.getInstance().handleError(e1);
		}
	}

}
