/*
 * iTunes Signature Maker
 * copyright (c) 2005 by Jason Freeman
 * 
 * For more information, please visit http://www.jasonfreeman.net/itsm/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 */

package net.jasonfreeman.itsm.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import net.jasonfreeman.itsm.ITSM;
import net.jasonfreeman.itsm.util.ErrorHandler;
import net.jasonfreeman.itsm.util.SettingsManager;

/**
 * Show the GPL license.
 * 
 * @author jason
 *
 */

public class LicensePanel extends JPanel implements ActionListener {

	JButton agree;
	ITSM applet;
	
    public LicensePanel(String resourcePath, ITSM applet) {
    		super();
        this.applet = applet;
        setBackground(Color.white);
        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
        
        add(Box.createVerticalGlue());
        add(Box.createRigidArea(new Dimension(0, 25)));
        
        JPanel labelPanel = new JPanel();
        labelPanel.setBackground(Color.white);
        labelPanel.setLayout(new BoxLayout(labelPanel, BoxLayout.X_AXIS));
        labelPanel.add(Box.createRigidArea(new Dimension(50, 0)));
        JLabel label = new JLabel("You must accept the terms of this license to continue.");
        label.setFont(label.getFont().deriveFont(Font.BOLD));
        labelPanel.add(label);
        labelPanel.add(Box.createHorizontalGlue());
        add(labelPanel);
        
        add(Box.createVerticalGlue());
        add(new HorizontalSeparator());
        add(Box.createVerticalGlue());
        
        JPanel htmlPanel = new JPanel();
        htmlPanel.setBackground(Color.white);
        htmlPanel.setLayout(new BoxLayout(htmlPanel, BoxLayout.X_AXIS));
        htmlPanel.add(Box.createRigidArea(new Dimension(50, 0)));
        JEditorPane info;
        try {
    			info = new JEditorPane(this.getClass().getResource(resourcePath));
        } catch (Exception e) {
    			info = new JEditorPane("text/plain", "Error: Could not locate info file.");
    		}
    		info.setEditable(false);
    		info.setOpaque(false);
    		info.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
    		JScrollPane editorScrollPane = new JScrollPane(info);
    		editorScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
    		editorScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    		editorScrollPane.setPreferredSize(new Dimension(400, 300));
    		htmlPanel.add(editorScrollPane);
    		htmlPanel.add(Box.createRigidArea(new Dimension(50, 0)));
    		info.setCaretPosition(0);
    		add(htmlPanel);
    		
    		add(Box.createVerticalGlue());
    		add(new HorizontalSeparator());
    		add(Box.createVerticalGlue());
    		
    		JPanel buttonPanel = new JPanel();
    		buttonPanel.setBackground(Color.white);
    		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
    		buttonPanel.add(Box.createHorizontalGlue());
    		agree = new JButton("I agree to the terms of this license.");
    		agree.addActionListener(this);
    		agree.setBackground(Color.white);
    		buttonPanel.add(agree);
    		buttonPanel.add(Box.createHorizontalGlue());
    		add(buttonPanel);
    		add(Box.createVerticalGlue());
    		
    		applet.setContentPane(this);
    		applet.validate();
    }

	public void actionPerformed(ActionEvent e) {
		try {
			if (e.getSource() == agree) {
	            // only add XML pane if we can't locate the file automatically
	            String xmlFile = SettingsManager.getInstance().getiTunesXmlFile();
	            File f = new File(xmlFile);
	            if (!f.exists()) {
	            		new iTunesXMLPanel(applet);
	            } else {
	                 new SongSelectionPanel(applet);
	            }
			}
		} catch (Throwable e1) {
			ErrorHandler.getInstance().handleError(e1);
		}
	}

}
