/*
 * iTunes Signature Maker
 * copyright (c) 2005 by Jason Freeman
 * 
 * For more information, please visit http://www.jasonfreeman.net/itsm/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 */

package net.jasonfreeman.itsm.gui;

import java.awt.Color;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;

import net.jasonfreeman.itsm.ITSM;

/**
 * Show progress bar and message during long operations.
 * 
 * @author jason
 *
 */

public class LoadingPanel extends JPanel implements ActionListener {
	
	JLabel label;
	JProgressBar prog;
	JButton cancel;
	ITSM applet;
	
	public LoadingPanel(ITSM applet, String message) {
		this(applet, message, false);
	}
	
	/**
	 * 
	 * @param applet
	 * @param message progress message to display
	 * @param showCancel if true, operation is cancel-able
	 */
	public LoadingPanel(ITSM applet, String message, boolean showCancel) {
		super();
		this.applet = applet;
		setBackground(Color.white);
        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
        add(Box.createVerticalGlue());
        
        JPanel textPanel = new JPanel();
        textPanel.setBackground(Color.white);
        textPanel.setLayout(new BoxLayout(textPanel, BoxLayout.X_AXIS));
        textPanel.add(Box.createHorizontalGlue());
        label = new JLabel(message);
        label.setFont(label.getFont().deriveFont(Font.BOLD));
        textPanel.add(label);
        textPanel.add(Box.createHorizontalGlue());
        add(textPanel);
        add(Box.createVerticalGlue());
        add(new HorizontalSeparator());
        add(Box.createVerticalGlue());
        add(Box.createVerticalGlue());
        add(Box.createVerticalGlue());
        add(Box.createVerticalGlue());
        add(Box.createVerticalGlue());
        
        JPanel progPanel = new JPanel();
        progPanel.setBackground(Color.white);
        progPanel.setLayout(new BoxLayout(progPanel, BoxLayout.X_AXIS));
        progPanel.add(Box.createHorizontalGlue());
        prog = new JProgressBar();
        progPanel.add(prog);
        progPanel.add(Box.createHorizontalGlue());
        add(progPanel);
        add(Box.createVerticalGlue());
        add(Box.createVerticalGlue());
        add(Box.createVerticalGlue());
        add(Box.createVerticalGlue());
        
        if (showCancel) {
        		add(new HorizontalSeparator());
        		add(Box.createVerticalGlue());
	        JPanel cancelPanel = new JPanel();
	        cancelPanel.setBackground(Color.white);
	        cancelPanel.setLayout(new BoxLayout(cancelPanel, BoxLayout.X_AXIS));
	        cancelPanel.add(Box.createHorizontalGlue());
	        cancel = new JButton("cancel");
	        cancel.setBackground(Color.white);
	        cancel.addActionListener(this);
	        cancelPanel.add(cancel);
	        cancelPanel.add(Box.createHorizontalGlue());
	        add(cancelPanel);
	        add(Box.createVerticalGlue());
        }

        applet.setContentPane(this);
        applet.validate();
	}
	
	public void setMessage(String message) {
		label.setText(message);
	}
	
	public JProgressBar getProgressBar() {
		return prog;
	}
	
	public void setProgress(int percent) {
		prog.setValue(percent);
	}

	/**
	 * Since cancel is only used for file upload, we go straight to that class
	 * to cancel it instead of adding a whole extra interface implementation.
	 */
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == cancel)
			applet.getRemoteDatabase().cancel();
	}
}
