/*
 * iTunes Signature Maker
 * copyright (c) 2005 by Jason Freeman
 * 
 * For more information, please visit http://www.jasonfreeman.net/itsm/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 */

package net.jasonfreeman.itsm.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.SwingUtilities;

import net.jasonfreeman.itsm.ITSM;
import net.jasonfreeman.itsm.util.ErrorHandler;
import net.jasonfreeman.itsm.util.SettingsManager;

/**
 * Show detailed progress of signature making. Singleton class!
 * 
 * @author jason
 *
 */

public class ProgressManager extends JPanel implements ActionListener {

	private static ProgressManager instance = null;

	private JLabel taskInfo, warningLabel;

	private JProgressBar percentInfo;

	private JProgressBar totalInfo;

	private ITSM applet;

	private int currentPercent = -1;

	public synchronized static ProgressManager getInstance() {
		if (instance == null)
			instance = new ProgressManager();
		return instance;
	}

	public ProgressManager() {
		super();
	}

	public void buildProgressDialog(ITSM applet) {
		this.applet = applet;
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		add(Box.createVerticalGlue());
		add(Box.createVerticalGlue());
		setBackground(Color.white);
		JPanel panel = new JPanel();
		panel.setBackground(Color.white);
		panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
		panel.add(Box.createHorizontalGlue());
		JLabel label = new JLabel(
				"Please be patient while iTSM creates your signature.");
		label.setFont(label.getFont().deriveFont(Font.BOLD));
		panel.add(label);
		panel.add(Box.createHorizontalGlue());
		add(panel);
		add(Box.createVerticalGlue());
		add(Box.createVerticalGlue());
		add(new HorizontalSeparator());
		add(Box.createVerticalGlue());
		add(Box.createVerticalGlue());

		panel = new JPanel();
		panel.setBackground(Color.white);
		panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
		panel.add(Box.createRigidArea(new Dimension(50, 0)));
		taskInfo = new JLabel("Please wait");
		panel.add(taskInfo);
		panel.add(Box.createHorizontalGlue());
		add(panel);
		add(Box.createVerticalGlue());

		panel = new JPanel();
		panel.setBackground(Color.white);
		panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
		panel.add(Box.createRigidArea(new Dimension(50, 0)));
		percentInfo = new JProgressBar(0, 100);
		percentInfo.setBackground(Color.white);
		percentInfo.setValue(0);
		percentInfo.setStringPainted(true);
		panel.add(percentInfo);
		panel.add(Box.createRigidArea(new Dimension(50, 0)));
		add(panel);
		add(Box.createVerticalGlue());
		add(Box.createVerticalGlue());

		panel = new JPanel();
		panel.setBackground(Color.white);
		panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
		panel.add(Box.createRigidArea(new Dimension(50, 0)));
		label = new JLabel("Total Progress");
		panel.add(label);
		panel.add(Box.createHorizontalGlue());
		add(panel);
		add(Box.createVerticalGlue());

		panel = new JPanel();
		panel.setBackground(Color.white);
		panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
		panel.add(Box.createRigidArea(new Dimension(50, 0)));
		totalInfo = new JProgressBar(0, 100);
		totalInfo.setBackground(Color.white);
		totalInfo.setValue(0);
		totalInfo.setStringPainted(true);
		panel.add(totalInfo);
		panel.add(Box.createRigidArea(new Dimension(50, 0)));
		add(panel);
		add(Box.createVerticalGlue());
		add(Box.createVerticalGlue());
		
		panel = new JPanel();
		panel.setBackground(Color.white);
		panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
		panel.add(Box.createHorizontalGlue());
		warningLabel = new JLabel("");
		panel.add(warningLabel);
		panel.add(Box.createHorizontalGlue());
		add(panel);
		add(Box.createVerticalGlue());
		add(Box.createVerticalGlue());
		
		add(new HorizontalSeparator());
		add(Box.createVerticalGlue());
		add(Box.createVerticalGlue());
		
		panel = new JPanel();
		panel.setBackground(Color.white);
		panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
		panel.add(Box.createHorizontalGlue());
		JButton cancel = new JButton("cancel");
		cancel.setBackground(Color.white);
		cancel.addActionListener(this);
		panel.add(cancel);
		panel.add(Box.createHorizontalGlue());
		add(panel);
		add(Box.createVerticalGlue());

		applet.setContentPane(this);
		applet.validate();
	}

	/**
	 *
	 * @param text name of current sub-task being executed
	 */
	public void setTaskInfo(final String text) {
		Runnable updateAComponent = new Runnable() {
			public void run() {
				try {
					taskInfo.setText(text);
				} catch (Throwable e) {
					ErrorHandler.getInstance().handleError(e);
				}
			}
		};
		SwingUtilities.invokeLater(updateAComponent);
	}

	/**
	 * 
	 * @param percent completion of current task
	 */
	public void setPercentInfo(final int percent) {
		if (percent == currentPercent)
			return;
		currentPercent = percent;
		Runnable updateAComponent = new Runnable() {
			public void run() {
				try {
					if (percent < 0)
						percentInfo.setValue(0);
					else
						percentInfo.setValue(percent);
				} catch (Throwable e) {
					ErrorHandler.getInstance().handleError(e);
				}
			}
		};
		SwingUtilities.invokeLater(updateAComponent);
	}

	/**
	 * 
	 * @param percent completion of entire process
	 */
	public void setTotalInfo(final int percent) {
		Runnable updateAComponent = new Runnable() {
			public void run() {
				try {
					totalInfo.setValue(percent);
				} catch (Throwable e) {
					ErrorHandler.getInstance().handleError(e);
				}
			}
		};
		SwingUtilities.invokeLater(updateAComponent);
	}

	/**
	 * Show warning that iTSM can't use files from iTunes Music Store.
	 *
	 */
	public void showProtectedWarning() {
		Runnable updateAComponent = new Runnable() {
			public void run() {
				try {
					warningLabel.setText("Note: Technical and legal restrictions prevent iTSM from including songs purchased from Apple's iTunes Music Store.");
				} catch (Throwable e) {
					ErrorHandler.getInstance().handleError(e);
				}
			}
		};
		SwingUtilities.invokeLater(updateAComponent);
	}
	
	public void actionPerformed(ActionEvent e) {
		try {
			cancel();
		} catch (Throwable e1) {
			ErrorHandler.getInstance().handleError(e1);
		}
	}

	private void cancel() {
		SettingsManager.getInstance().setProcessing(false);
		setVisible(false);
		new CanceledPanel(applet);
	}

	public static void shutdown() {
		instance = null;
	}
}
