/*
 * iTunes Signature Maker
 * copyright (c) 2005 by Jason Freeman
 * 
 * For more information, please visit http://www.jasonfreeman.net/itsm/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 */

package net.jasonfreeman.itsm.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import net.jasonfreeman.itsm.ITSM;
import net.jasonfreeman.itsm.util.ErrorHandler;
import net.jasonfreeman.itsm.util.SettingsManager;

/**
 * Confirm where the signature will be saved on the local disk.
 * 
 * @author jason
 *
 */

public class SavePanel extends JPanel implements ActionListener {

	private JLabel filePath;

	private JButton select, continueButton;

	private SettingsManager settings = SettingsManager.getInstance();
	
	private boolean complete;
	
	private ITSM applet;

	public SavePanel(ITSM applet) {
		this.applet = applet;
		String saveFile;
		File saveDir = new File(settings.getOutDir());
		// double check that default directory will work
		if (saveDir.exists() && saveDir.canWrite()) {
			saveFile = settings.getOutFile();
			complete = true;
		} else {
			saveFile = "";
			complete = false;
		}
		setBackground(Color.white);
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		add(Box.createVerticalGlue());

        JPanel labelPanel = new JPanel();
        labelPanel.setBackground(Color.white);
        labelPanel.setLayout(new BoxLayout(labelPanel, BoxLayout.X_AXIS));
        labelPanel.add(Box.createRigidArea(new Dimension(50, 0)));
        JLabel label = new JLabel("Specify where the iTunes signature should be saved on your hard disk.");
        label.setFont(label.getFont().deriveFont(Font.BOLD));
        labelPanel.add(label);
        labelPanel.add(Box.createHorizontalGlue());
        add(labelPanel);
		
		add(Box.createVerticalGlue());
		add(new HorizontalSeparator());
		add(Box.createVerticalGlue());
        
		JPanel infoPanel = new JPanel();
		infoPanel.setBackground(Color.white);
		infoPanel.setLayout(new BoxLayout(infoPanel, BoxLayout.X_AXIS));
		infoPanel.add(Box.createRigidArea(new Dimension(50, 0)));
		label = new JLabel("Location to save iTunes signature file:");
		infoPanel.add(label);
		infoPanel.add(Box.createHorizontalGlue());
		add(infoPanel);
		add(Box.createVerticalGlue());

		JPanel selectPanel = new JPanel();
		selectPanel.setBackground(Color.white);
		selectPanel.setLayout(new BoxLayout(selectPanel, BoxLayout.X_AXIS));
		selectPanel.add(Box.createRigidArea(new Dimension(50, 0)));
		filePath = new JLabel(saveFile);
		selectPanel.add(filePath);
		selectPanel.add(Box.createHorizontalGlue());
		add(selectPanel);
		add(Box.createVerticalGlue());
		
		JPanel choosePanel = new JPanel();
		choosePanel.setBackground(Color.white);
		choosePanel.setLayout(new BoxLayout(choosePanel, BoxLayout.X_AXIS));
		choosePanel.add(Box.createRigidArea(new Dimension(50, 0)));
		select = new JButton("Select...");
		select.addActionListener(this);
		select.setBackground(Color.white);
		choosePanel.add(select);
		choosePanel.add(Box.createHorizontalGlue());
		add(choosePanel);
		add(Box.createVerticalGlue());
		add(new HorizontalSeparator());
		add(Box.createVerticalGlue());
		
        JPanel continuePanel = new JPanel();
        continuePanel.setBackground(Color.white);
        continuePanel.setLayout(new BoxLayout(continuePanel, BoxLayout.X_AXIS));
        continuePanel.add(Box.createHorizontalGlue());
        continueButton = new JButton("create my signature");
        continueButton.setBackground(Color.white);
        continueButton.addActionListener(this);
        continueButton.setEnabled(complete);
        continuePanel.add(continueButton);
        continuePanel.add(Box.createHorizontalGlue());
        add(continuePanel);
        
		add(Box.createVerticalGlue());
        
        applet.setContentPane(this);
        applet.validate();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		try {
			if (e.getSource() == select) {
				selectFile();
			} else if (e.getSource() == continueButton) {
		         Runnable t = new Runnable() {
		             public void run() {
		                 applet.finish();
		             }
		          };
		          new Thread(t).start();
			}
		} catch (Throwable e1) {
			ErrorHandler.getInstance().handleError(e1);
		}
	}

	private void selectFile() {
		File file = null;
		JFileChooser fc = new JFileChooser();
		fc.setSelectedFile(new File(settings.getOutFile()));
		int returnVal = fc.showSaveDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			file = fc.getSelectedFile();
			if (!file.getName().endsWith(".wav")) {
				int val = JOptionPane
						.showConfirmDialog(
								this,
								"iTSM saves signatures in WAV format. OK to append \".wav\" to filename?",
								"File name extension",
								JOptionPane.OK_CANCEL_OPTION,
								JOptionPane.WARNING_MESSAGE);
				if (val == JOptionPane.OK_OPTION) {
					file = new File(file.getAbsolutePath() + ".wav");
				} else
					return;
			}
			if (file.exists()) {
				int val = JOptionPane.showConfirmDialog(this,
						"File exists, OK to overwrite?",
						"Overwrite existing file",
						JOptionPane.OK_CANCEL_OPTION,
						JOptionPane.WARNING_MESSAGE);
				if (val == JOptionPane.CANCEL_OPTION)
					return;
			}
			if (!isValidFileName(file)) {
        			JOptionPane.showMessageDialog(this,
        				"Invalid file! Your filename cannot contain colons.",
						"File error",
        		    JOptionPane.ERROR_MESSAGE);
        			return;
			}
			if (!isValidFileLocation(file)) {
    				JOptionPane.showMessageDialog(this,
        				"You do not have permission to write to that directory.",
						"File error",
        		    JOptionPane.ERROR_MESSAGE);
        			return;
			}
			settings.setOutFile(file.getAbsolutePath());
			filePath.setText(file.getAbsolutePath());
			continueButton.setEnabled(true);
		}
	}

	private boolean isValidFileName(File f) {
		return (f.getName().indexOf(':') == -1);
	}
	
	private boolean isValidFileLocation(File f) {
		if (f.exists())
			return f.canWrite();
		boolean created;
		try {
			created = f.createNewFile();
		} catch (IOException e) {
			return false;
		}
		boolean canWrite = f.canWrite();
		f.delete();
		return (created && canWrite);
	}
}
