/*
 * iTunes Signature Maker
 * copyright (c) 2005 by Jason Freeman
 * 
 * For more information, please visit http://www.jasonfreeman.net/itsm/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 */

package net.jasonfreeman.itsm.gui;

import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import net.jasonfreeman.itsm.ITSM;
import net.jasonfreeman.itsm.util.ErrorHandler;
import net.jasonfreeman.itsm.util.SettingsManager;

/**
 * Let users upload their signature to the gallery.
 * 
 * @author jason
 *
 */

public class SharePanel extends JPanel implements ActionListener {

	JButton shareButton, cancelButton;
	JTextField textField;
	ITSM applet;
	File file;
	Container contentPane;
	
	public SharePanel(ITSM applet) {
		super();
		this.applet = applet;
		setBackground(Color.white);
        SettingsManager settings = SettingsManager.getInstance();
        
        file = new File(settings.getOutFile());

        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

        add(Box.createVerticalGlue());
        add(Box.createVerticalGlue());
        
        JPanel textPanel = new JPanel();
        textPanel.setBackground(Color.white);
        textPanel.setLayout(new BoxLayout(textPanel, BoxLayout.X_AXIS));
        textPanel.add(Box.createRigidArea(new Dimension(50, 0)));
        JLabel label = new JLabel("Please enter your name (to identify the signature):");
        label.setFont(label.getFont().deriveFont(Font.BOLD));
        textPanel.add(label);
        textPanel.add(Box.createHorizontalGlue());
        add(textPanel);
        add(Box.createVerticalGlue());
        add(new HorizontalSeparator());
        add(Box.createVerticalGlue());
        
        JPanel namePanel = new JPanel();
        namePanel.setBackground(Color.white);
        namePanel.setLayout(new BoxLayout(namePanel, BoxLayout.X_AXIS));
        namePanel.add(Box.createRigidArea(new Dimension(50,0)));
        String username = System.getProperty("user.name");
        if (username.length() == 0)
        		username = "Anonymous";
        textField = new JTextField(username, 10);
        textField.setMaximumSize(new Dimension(100, 40)); // need this hack on Mac
        namePanel.add(textField);
        namePanel.add(Box.createHorizontalGlue());
        add(namePanel);
        add(Box.createVerticalGlue());
        add(Box.createVerticalGlue());
        
        JPanel sharePanel = new JPanel();
        sharePanel.setBackground(Color.white);
        sharePanel.setLayout(new BoxLayout(sharePanel, BoxLayout.X_AXIS));
        sharePanel.add(Box.createHorizontalGlue());
        sharePanel.add(Box.createHorizontalGlue());
        shareButton = new JButton("share");
        shareButton.addActionListener(this);
        shareButton.setBackground(Color.white);
        sharePanel.add(shareButton);
        sharePanel.add(Box.createHorizontalGlue());
        cancelButton = new JButton("cancel");
        cancelButton.addActionListener(this);
        cancelButton.setBackground(Color.white);
        sharePanel.add(cancelButton);
        sharePanel.add(Box.createHorizontalGlue());
        sharePanel.add(Box.createHorizontalGlue());
        add(sharePanel);
        add(Box.createVerticalGlue());
        add(new HorizontalSeparator());
        add(Box.createVerticalGlue());
        
        textPanel = new JPanel();
        textPanel.setBackground(Color.white);
        textPanel.setLayout(new BoxLayout(textPanel, BoxLayout.X_AXIS));
        textPanel.add(Box.createRigidArea(new Dimension(50, 0)));
        label = new JLabel("Press the share button to upload your signature to the gallery.");
        label.setFont(label.getFont().deriveFont(Font.BOLD));
        textPanel.add(label);
        textPanel.add(Box.createHorizontalGlue());
        add(textPanel);
        add(Box.createVerticalGlue());
        
        textPanel = new JPanel();
        textPanel.setBackground(Color.white);
        textPanel.setLayout(new BoxLayout(textPanel, BoxLayout.X_AXIS));
        textPanel.add(Box.createRigidArea(new Dimension(50, 0)));
        label = new JLabel("To browse the signature gallery, click the gallery link at the top of this window.");
        textPanel.add(label);
        textPanel.add(Box.createHorizontalGlue());
        add(textPanel);
        add(Box.createVerticalGlue());
        
        textPanel = new JPanel();
        textPanel.setBackground(Color.white);
        textPanel.setLayout(new BoxLayout(textPanel, BoxLayout.X_AXIS));
        textPanel.add(Box.createRigidArea(new Dimension(50, 0)));
        label = new JLabel("(By pressing share, you agree for your signature to be distributed online and on other media.)");
        textPanel.add(label);
        textPanel.add(Box.createHorizontalGlue());
        add(textPanel);
        add(Box.createVerticalGlue());
        add(Box.createVerticalGlue());
        contentPane = applet.getContentPane();
        applet.setContentPane(this);
        applet.validate();
        textField.requestFocus();
        textField.selectAll();
	}
	
	public void actionPerformed(ActionEvent e) {
		try {
			if (e.getSource() == shareButton) {
				String name = textField.getText();
				if (name.length() > 0) {
					final ITSM applet = this.applet;
					final String theName = name;
					final LoadingPanel loading = new LoadingPanel(applet, "Please be patient while ITSM sends your signature...", true);
					Runnable r = new Runnable() {
						public void run() {
							try {
								applet.getRemoteDatabase().sendFile(theName, file, loading.getProgressBar());
								if (!applet.getRemoteDatabase().checkAndResetCanceled()) {
									loading.setMessage("Finished.");
									loading.getProgressBar().setValue(loading.getProgressBar().getMaximum());
									try { Thread.sleep(1000); } catch (InterruptedException e) { }
								}
								applet.setContentPane(contentPane);
							} catch (Throwable e) {
								ErrorHandler.getInstance().handleError(e);
							}
						}
					};
					new Thread(r).start();
				} else
					JOptionPane.showMessageDialog(this, "You must enter a name for your signature.");
			} else if (e.getSource() == cancelButton) {
				applet.setContentPane(contentPane);
			}
		} catch (Throwable e1) {
			ErrorHandler.getInstance().handleError(e1);
		}
	}	
}
