/*
 * iTunes Signature Maker
 * copyright (c) 2005 by Jason Freeman
 * 
 * For more information, please visit http://www.jasonfreeman.net/itsm/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 */

package net.jasonfreeman.itsm.mixing;

import net.jasonfreeman.itsm.util.ErrorHandler;
import net.jasonfreeman.itsm.util.SettingsManager;
import quicktime.QTException;
import quicktime.io.IOConstants;
import quicktime.io.QTFile;
import quicktime.io.QTIOException;
import quicktime.std.StdQTConstants;
import quicktime.std.StdQTException;
import quicktime.std.comp.ComponentDescription;
import quicktime.std.comp.ComponentIdentifier;
import quicktime.std.movies.Movie;
import quicktime.std.movies.media.SoundDescription;
import quicktime.std.qtcomponents.MovieExporter;

/**
 * Export the mixed signature to a WAV file.
 * 
 * @author jason
 *
 */

public class Exporter {

public static boolean export(Movie mov) throws QTException {
		mov.setProgressProc();
		
		QTFile saveFile = new QTFile(SettingsManager.getInstance()
				.getOutFile());
		
		ComponentDescription cd =
			new ComponentDescription(
				StdQTConstants.movieExportType,
				StdQTConstants.kQTFileTypeWave,
				StdQTConstants.soundMediaType,
				0,
				0);
		ComponentIdentifier ci = ComponentIdentifier.find(cd);
		MovieExporter exp = null;
		try {
			exp = new MovieExporter(ci);
		} catch (NullPointerException e) {
			ErrorHandler.getInstance().handleError(e, "We were unable to find the Quicktime component necessary to save your signature to disk. We suggest you upgrade to the latest version of Quicktime at www.apple.com/quicktime.");
			return false;
		}
		SoundDescription sd = new SoundDescription(0);
		sd.setNumberOfChannels(2);
		sd.setSampleRate(44100);
		sd.setSampleSize(16);
		sd.lock();
		exp.setSampleDescription(sd, StdQTConstants.soundMediaType);

		
		//MovieExporter exp = new MovieExporter(QTUtils.toOSType("WAVE"));

		// sleep for a second to avoid possible crash
		try {
			Thread.sleep(1000);
		} catch (InterruptedException e) {
		}

		try {
			mov.convertToFile(null, saveFile, StdQTConstants.kQTFileTypeWave,
				StdQTConstants.kMoviePlayer, IOConstants.smSystemScript,
				StdQTConstants.createMovieFileDeleteCurFile
						| StdQTConstants.movieFileSpecValid, exp);
			// sleep for a second to avoid possible crash
			try { Thread.sleep(1000); } catch (InterruptedException e) { }
		} catch (QTIOException e) {
			ErrorHandler.getInstance().handleError(e, "We could not save your signature to the location you specified because of a file write error. Aborting!");
			sd.unlock();
			return false;
		} catch (StdQTException e) {
			if (e.toString().indexOf("progressProcAborted") == -1)
				ErrorHandler.getInstance().handleError(e, "An error occurred while trying to save your signature to disk. Sorry!");
			sd.unlock();
			return false;
		}
		sd.unlock();
		return true;

		// temp save debugging movie
		
		 /*QTFile file = new QTFile (new File
				 (SettingsManager.getInstance().getOutFile() + ".mov"));
		 if (!file.exists( )) { 
			 file.createMovieFile(StdQTConstants.kMoviePlayer,
					 StdQTConstants.createMovieFileDontCreateResFile);
			 } 
		 //OpenMovieFile outFile = OpenMovieFile.asWrite(file);
		 //mov.updateResource(outFile, StdQTConstants.movieInDataForkResID, null);
		 mov.flatten(StdQTConstants.flattenAddMovieToDataFork |
				 StdQTConstants.flattenForceMovieResourceBeforeMovieData, file, 
				 StdQTConstants.kMoviePlayer, 
				 IOConstants.smSystemScript, 
				 StdQTConstants.createMovieFileDeleteCurFile, // createQTFileFlags
				 StdQTConstants.movieInDataForkResID, // resID
				 null); // resName
		 return true;*/
		 
	}
}
