/*
 * iTunes Signature Maker
 * copyright (c) 2005 by Jason Freeman
 * 
 * For more information, please visit http://www.jasonfreeman.net/itsm/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 */

package net.jasonfreeman.itsm.selection;

/**
 * Represents a single song within the iTunes Music Library.
 */

import java.io.File;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLDecoder;

import net.jasonfreeman.itsm.util.ErrorHandler;

import quicktime.QTException;
import quicktime.io.OpenMovieFile;
import quicktime.io.QTFile;
import quicktime.std.movies.Movie;


public class Song {
    private String location = "";

    private int playCount = 0;

    private String lastPlayed = "";

    private String dateCreated = "";
    
    private String name = "";
    
    private String genre = "";
    
    private String kind = "";
    
    private String album = "";
    
    private String artist = "";

    private int rating = 0;

    public Song() {

    }

    public Song(String location, int playCount, String lastPlayed,
            String dateCreated, String album, String artist) {
        setLocation(location);
        setPlayCount(playCount);
        setLastPlayed(lastPlayed);
        setDateCreated(dateCreated);
        setAlbum(album);
        setArtist(artist);
    }

    public Movie openAsMovie() throws QTException, MalformedURLException {
        URL url = new URL(getLocation());
        String path;
        try {
        		// have to do in 2 steps to support Java 1.3.1
        		path = URLDecoder.decode(url.getFile());
			path = new String(path.getBytes(), "UTF-8");
		} catch (UnsupportedEncodingException e) {
			ErrorHandler.getInstance().handleError(e);
			return null;
		}
		QTFile qtf = new QTFile(new File(path));
        Movie mov = null;
        OpenMovieFile omf = OpenMovieFile.asRead(qtf);
        mov = Movie.fromFile(omf);
        omf.close();
        return mov;
    }

    /**
     * @return Returns the dateCreated.
     */
    public String getDateCreated() {
        return dateCreated;
    }

    /**
     * @param dateCreated
     *            The dateCreated to set.
     */
    public void setDateCreated(String dateCreated) {
        this.dateCreated = dateCreated;
    }

    /**
     * @return Returns the lastPlayed.
     */
    public String getLastPlayed() {
        return lastPlayed;
    }

    /**
     * @param lastPlayed
     *            The lastPlayed to set.
     */
    public void setLastPlayed(String lastPlayed) {
        this.lastPlayed = lastPlayed;
    }

    /**
     * @return Returns the location.
     */
    public String getLocation() {
        return location;
    }

    /**
     * @param location
     *            The location to set.
     */
    public void setLocation(String location) {
        this.location = location;
    }

    /**
     * @return Returns the playCount.
     */
    public int getPlayCount() {
        return playCount;
    }

    /**
     * @param playCount
     *            The playCount to set.
     */
    public void setPlayCount(int playCount) {
        this.playCount = playCount;
    }

    /**
     * @return Returns the rating.
     */
    public int getRating() {
        return rating;
    }

    /**
     * @param rating
     *            The rating to set.
     */
    public void setRating(int rating) {
        this.rating = rating;
    }

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getGenre() {
		return genre;
	}

	public void setGenre(String genre) {
		this.genre = genre;
	}

	public String getKind() {
		return kind;
	}

	public void setKind(String kind) {
		this.kind = kind;
	}

	public String getAlbum() {
		return album;
	}

	public void setAlbum(String album) {
		this.album = album;
	}

	public String getArtist() {
		return artist;
	}

	public void setArtist(String artist) {
		this.artist = artist;
	}
    
    
}