/*
 * iTunes Signature Maker
 * copyright (c) 2005 by Jason Freeman
 * 
 * For more information, please visit http://www.jasonfreeman.net/itsm/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 */

package net.jasonfreeman.itsm.util;

import java.io.PrintWriter;
import java.io.StringWriter;

import javax.swing.SwingUtilities;

import net.jasonfreeman.itsm.ITSM;
import net.jasonfreeman.itsm.gui.ErrorPanel;

/**
 * Handles all Java exceptions and other errors to display them to user and
 * report them to the server. Singleton class!
 * 
 * @author jason
 *
 */

public class ErrorHandler {

    private static ErrorHandler instance = null;
    private static ITSM applet = null;

    public void setApplet(ITSM applet) {
    		this.applet = applet;
    }
    
    public void handleError(Throwable t, String s) {
    		t.printStackTrace();
    		String message = s + "\n\nStack Trace:\n" +
			getStackTrace(t);
    		showError(message);
    		if (applet != null) {
    			RemoteDatabase rd = applet.getRemoteDatabase();
    			if (rd != null) 
    				rd.sendParameter("error", message);
    		}
    }
    
    public void handleError(Throwable t) {
    		t.printStackTrace();
    		String message = getStackTrace(t);
    		showError(message);
    		if (applet != null) {
    			RemoteDatabase rd = applet.getRemoteDatabase();
    			if (rd != null) 
    				rd.sendParameter("error", message);
    		}
    }
    
    public void handleWarning(String message) {
    		showError(message);
    		// don't log to server!
    }
    
    public synchronized static ErrorHandler getInstance() {
        if (instance == null)
            instance = new ErrorHandler();
        return instance;
    }
    
    public static void shutdown() {
    		instance = null;
    }
    
	/**
	 * 
	 * @param e the exception whose trace we want to get
	 * @return a string representing the stack trace
	 */
	private String getStackTrace(Throwable e) {
		StringWriter sw = new StringWriter();
		PrintWriter pw = new PrintWriter(sw);
		e.printStackTrace(pw);		
		return(sw.toString());
	}
	
	private void showError(final String s) {
        final ITSM applet = this.applet;
        Runnable updateAComponent = new Runnable() {
            public void run() { 
            		new ErrorPanel(applet, s);
            	}
        };
        SwingUtilities.invokeLater(updateAComponent); 
	}
}
