/*
 * iTunes Signature Maker
 * copyright (c) 2005 by Jason Freeman
 * 
 * For more information, please visit http://www.jasonfreeman.net/itsm/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 */

package net.jasonfreeman.itsm.util;

import java.io.File;
import java.net.UnknownHostException;
import java.util.LinkedList;

/**
 * Separate thread to queue and handle HTTP server requests.
 * 
 * @author jason
 *
 */

public class ServerConnectionThread extends Thread {

	String ssid = null;
	LinkedList actionQ = new LinkedList();
	boolean stop = false;
	boolean stopped = false;
	HttpPostMethod post = null;
	ServerAction act = null;
	
	public void run() {
		try {
			while (!stop) {
				if (actionQ.isEmpty()) {
					try {
						Thread.sleep(1000);
					} catch (InterruptedException e) {
					}
				} else {
					act = (ServerAction) actionQ.removeFirst();
					act.status = ServerAction.STATUS_RUNNING;
					String[] names = act.names;
					String[] values = act.values;
					File[] files = act.files;
					// update ssid just in case of threading issue...
					for (int i=0; i < names.length; i++) {
						if (names[i].compareTo("ssid") == 0)
							values[i] = ssid;
					}
					// post the data
					post = new HttpPostMethod(files, names,
							values, SettingsManager.getInstance().getHostName()
									+ SettingsManager.getInstance()
											.getLogPath());
					if (act.progressBar != null)
						post.setProgressPanel(act.progressBar);
					post.setWaitForResponse(act.waitForResponse);
					post.run();
					if (stop)
						return;
					if (post.getException() != null) {
						ssid = null;
						// unknownhostexception means server is down or we're not
						// connected to the Internet
						if (!(post.getException() instanceof UnknownHostException))
							ErrorHandler.getInstance().handleError(
								post.getException(),
								"Could not connect with server");
						act.status = ServerAction.STATUS_FAILED;
					} else {
						if (act.waitForResponse)
							act.response = post.getServerOutput().toString();
						if (act.type == ServerAction.TYPE_INIT) {
							int ssidStart = act.response.lastIndexOf("id=") + 3;
							int ssidEnd = act.response.indexOf(' ', ssidStart);
							ssid = act.response.substring(ssidStart, ssidEnd);
							if (!isNumeric(ssid))
								ssid = null;
						}
						act.status = ServerAction.STATUS_DONE;
					}
					post.close();
					if (ssid == null)
						stop = true;
				}
			}
		} catch (Throwable e) {
			ssid = null;
			ErrorHandler.getInstance().handleError(e,
					"Could not connect with server");
		}
		stopped = true;
	}

	public String getSsid() {
		return ssid;
	}
	
	public void addAction(ServerAction act) {
		if (!stop)
			actionQ.add(act);
		else
			act.status = ServerAction.STATUS_ABORTED;
	}
	
	public void cancelEverything() {
		while (!actionQ.isEmpty()) {
			ServerAction act = (ServerAction) actionQ.removeFirst();
			act.status = ServerAction.STATUS_ABORTED;
		}
		if (post != null) {
			post.stopUpload();
		}
		if ((act != null) && (act.status != ServerAction.STATUS_DONE))
			act.status = ServerAction.STATUS_ABORTED;
	}
	
	public void shutdown(boolean wait) {
		stop = true;
		while (wait && !stopped) {
			try { Thread.sleep(100); } catch (InterruptedException e) { }
		}
	}
	
	private static final boolean isNumeric(final String s) {
		  final char[] numbers = s.toCharArray();
		  for (int x = 0; x < numbers.length; x++) {      
		    final char c = numbers[x];
		    if ((c >= '0') && (c <= '9')) continue;
		    return false; // invalid
		  }
		  return true; // valid
	}
	
}
