/*
 * iTunes Signature Maker
 * copyright (c) 2005 by Jason Freeman
 * 
 * For more information, please visit http://www.jasonfreeman.net/itsm/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * This source file adapted from:
 * code by Kristjan Varnik for the auracle project (http://auracle.org).
 * 
 */
package net.jasonfreeman.itsm.util;

import java.net.URL;

import javax.swing.JApplet;

/**
 * 
 * Check system compatibility and redirect to another web page based on results.
 * 
 */
public class SystemCheckApplet extends JApplet {

	public static String TARGET_FRAME = "_self";
	
	public void init() {
		String urlprefix = "";
		try {
			urlprefix = this.getParameter("urlprefix");
		} catch (NullPointerException e) {
			System.out.println("The url parameter was not found");
			e.printStackTrace();
		}
		if (!isOSFound()) {
			try {
				URL url = new URL(urlprefix + "unsupported.html");
				this.getAppletContext().showDocument(url, TARGET_FRAME);
			} catch (Throwable e) {
				System.out.println("Url could not be loaded");
				e.printStackTrace();
			}
		} else if (!isJavaFound()) {
			try {
				URL url = new URL(urlprefix + "upgradejava.html");
				this.getAppletContext().showDocument(url, TARGET_FRAME);
			} catch (Throwable e) {
				System.out.println("Url could not be loaded");
				e.printStackTrace();
			}
		} else if (!isTrusted()) {
			try {
				URL url = new URL(urlprefix + "security.html");
				this.getAppletContext().showDocument(url, TARGET_FRAME);
			} catch (Throwable e) {
				System.out.println("Url could not be loaded");
				e.printStackTrace();
			}
		} else if (!isQTJClassesFound()) {
			if (System.getProperty("os.name").startsWith("Windows")
					&& getParameter("fixwinqtj").equals("true")) {
				try {
					URL url;
					if (repairWinQTJ()) {
						url = new URL(urlprefix + "checkqt.html");
					} else {
						url = new URL(urlprefix + "upgradeqt.html");
					}
					this.getAppletContext().showDocument(url, TARGET_FRAME);
				} catch (Throwable e) {
					System.out.println("Url could not be loaded");
					e.printStackTrace();
				}
			} else {
				try {
					URL url = new URL(urlprefix + "upgradeqt.html");
					this.getAppletContext().showDocument(url, TARGET_FRAME);
				} catch (Throwable e) {
					System.out.println("Url could not be loaded");
					e.printStackTrace();
				}
			}
		} else if (!isQTJVersionOK()) {
			try {
				URL url = new URL(urlprefix + "upgradeqt.html");
				this.getAppletContext().showDocument(url, TARGET_FRAME);
			} catch (Throwable e) {
				System.out.println("Url could not be loaded");
				e.printStackTrace();
			}
		} else
			try {
				URL url = new URL(urlprefix + "loaditsm.html");
				this.getAppletContext().showDocument(url, TARGET_FRAME);
			} catch (Throwable e) {
				System.out.println("Url could not be loaded");
				e.printStackTrace();
			}
	}

	private boolean isTrusted() {
		try {
			SecurityManager sm = System.getSecurityManager();
			if (sm != null) {
				// if we can't read user home directory
				// then sandbox is in place
				sm.checkRead(System.getProperty("user.home"));
			}
		} catch (SecurityException e) {
			return false;
		}
		return true;
	}

	/**
	 * @return true if QTJ is installed
	 */
	private boolean isQTJClassesFound() {
		try {
			Class.forName("quicktime.util.QTBuild");
			return true;
		} catch (Exception e) {
			return false;
		}
	}

	private boolean repairWinQTJ() {
		try {
			return ClassPathChecker.fixQTJInstallation();
		} catch (Exception e) {
			return (false);
		}
	}

	private boolean isQTJVersionOK() {
		try {
			return QTJavaCheck.checkQTJVersion();
		} catch (Exception e) {
			return false;
		}
	}

	private boolean isJavaFound() {
		return QTJavaCheck.checkJavaVersion();
	}

	private boolean isOSFound() {
		return QTJavaCheck.checkOS();
	}
}